@@ -4,30 +4,23 @@
 
 //! Items specific to the [`Runtime`](crate::Runtime)'s webview.
 
-use crate::{window::DetachedWindow, Icon};
+use crate::{menu::Menu, window::DetachedWindow, Icon};
 
-#[cfg(feature = "menu")]
-use crate::{menu::Menu, MenuId};
-
-use serde::Deserialize;
-use serde_json::Value as JsonValue;
 use tauri_utils::config::{WindowConfig, WindowUrl};
 
 #[cfg(windows)]
-use winapi::shared::windef::HWND;
+use windows::Win32::Foundation::HWND;
 
-use std::{collections::HashMap, path::PathBuf};
-
-type UriSchemeProtocol =
-  dyn Fn(&str) -> Result<Vec<u8>, Box<dyn std::error::Error>> + Send + Sync + 'static;
+use std::{fmt, path::PathBuf};
 
 /// The attributes used to create an webview.
+#[derive(Debug)]
 pub struct WebviewAttributes {
   pub url: WindowUrl,
   pub initialization_scripts: Vec<String>,
   pub data_directory: Option<PathBuf>,
-  pub uri_scheme_protocols: HashMap<String, Box<UriSchemeProtocol>>,
   pub file_drop_handler_enabled: bool,
+  pub clipboard: bool,
 }
 
 impl WebviewAttributes {
@@ -37,63 +30,47 @@ impl WebviewAttributes {
       url,
       initialization_scripts: Vec::new(),
       data_directory: None,
-      uri_scheme_protocols: Default::default(),
       file_drop_handler_enabled: true,
+      clipboard: false,
     }
   }
 
   /// Sets the init script.
+  #[must_use]
   pub fn initialization_script(mut self, script: &str) -> Self {
     self.initialization_scripts.push(script.to_string());
     self
   }
 
   /// Data directory for the webview.
+  #[must_use]
   pub fn data_directory(mut self, data_directory: PathBuf) -> Self {
     self.data_directory.replace(data_directory);
     self
   }
 
-  /// Whether the webview URI scheme protocol is defined or not.
-  pub fn has_uri_scheme_protocol(&self, name: &str) -> bool {
-    self.uri_scheme_protocols.contains_key(name)
-  }
-
-  /// Registers a webview protocol handler.
-  /// Leverages [setURLSchemeHandler](https://developer.apple.com/documentation/webkit/wkwebviewconfiguration/2875766-seturlschemehandler) on macOS,
-  /// [AddWebResourceRequestedFilter](https://docs.microsoft.com/en-us/dotnet/api/microsoft.web.webview2.core.corewebview2.addwebresourcerequestedfilter?view=webview2-dotnet-1.0.774.44) on Windows
-  /// and [webkit-web-context-register-uri-scheme](https://webkitgtk.org/reference/webkit2gtk/stable/WebKitWebContext.html#webkit-web-context-register-uri-scheme) on Linux.
-  ///
-  /// # Arguments
-  ///
-  /// * `uri_scheme` The URI scheme to register, such as `example`.
-  /// * `protocol` the protocol associated with the given URI scheme. It's a function that takes an URL such as `example://localhost/asset.css`.
-  pub fn register_uri_scheme_protocol<
-    N: Into<String>,
-    H: Fn(&str) -> Result<Vec<u8>, Box<dyn std::error::Error>> + Send + Sync + 'static,
-  >(
-    mut self,
-    uri_scheme: N,
-    protocol: H,
-  ) -> Self {
-    let uri_scheme = uri_scheme.into();
-    self
-      .uri_scheme_protocols
-      .insert(uri_scheme, Box::new(move |data| (protocol)(data)));
-    self
-  }
-
   /// Disables the file drop handler. This is required to use drag and drop APIs on the front end on Windows.
+  #[must_use]
   pub fn disable_file_drop_handler(mut self) -> Self {
     self.file_drop_handler_enabled = false;
     self
   }
+
+  /// Enables clipboard access for the page rendered on **Linux** and **Windows**.
+  ///
+  /// **macOS** doesn't provide such method and is always enabled by default,
+  /// but you still need to add menu item accelerators to use shortcuts.
+  #[must_use]
+  pub fn enable_clipboard_access(mut self) -> Self {
+    self.clipboard = true;
+    self
+  }
 }
 
 /// Do **NOT** implement this trait except for use in a custom [`Runtime`](crate::Runtime).
 ///
 /// This trait is separate from [`WindowBuilder`] to prevent "accidental" implementation.
-pub trait WindowBuilderBase: Sized {}
+pub trait WindowBuilderBase: fmt::Debug + Sized {}
 
 /// A builder for all attributes related to a single webview.
 ///
@@ -107,57 +84,76 @@ pub trait WindowBuilder: WindowBuilderBase {
   fn with_config(config: WindowConfig) -> Self;
 
   /// Sets the menu for the window.
-  #[cfg(feature = "menu")]
-  #[cfg_attr(doc_cfg, doc(cfg(feature = "menu")))]
-  fn menu<I: MenuId>(self, menu: Menu<I>) -> Self;
+  #[must_use]
+  fn menu(self, menu: Menu) -> Self;
 
   /// Show window in the center of the screen.
+  #[must_use]
   fn center(self) -> Self;
 
   /// The initial position of the window's.
+  #[must_use]
   fn position(self, x: f64, y: f64) -> Self;
 
   /// Window size.
+  #[must_use]
   fn inner_size(self, min_width: f64, min_height: f64) -> Self;
 
   /// Window min inner size.
+  #[must_use]
   fn min_inner_size(self, min_width: f64, min_height: f64) -> Self;
 
   /// Window max inner size.
-  fn max_inner_size(self, min_width: f64, min_height: f64) -> Self;
+  #[must_use]
+  fn max_inner_size(self, max_width: f64, max_height: f64) -> Self;
 
   /// Whether the window is resizable or not.
+  #[must_use]
   fn resizable(self, resizable: bool) -> Self;
 
   /// The title of the window in the title bar.
+  #[must_use]
   fn title<S: Into<String>>(self, title: S) -> Self;
 
   /// Whether to start the window in fullscreen or not.
+  #[must_use]
   fn fullscreen(self, fullscreen: bool) -> Self;
 
   /// Whether the window will be initially hidden or focused.
+  #[must_use]
   fn focus(self) -> Self;
 
   /// Whether the window should be maximized upon creation.
+  #[must_use]
   fn maximized(self, maximized: bool) -> Self;
 
   /// Whether the window should be immediately visible upon creation.
+  #[must_use]
   fn visible(self, visible: bool) -> Self;
 
   /// Whether the the window should be transparent. If this is true, writing colors
   /// with alpha values different than `1.0` will produce a transparent window.
+  #[cfg(any(not(target_os = "macos"), feature = "macos-private-api"))]
+  #[cfg_attr(
+    doc_cfg,
+    doc(cfg(any(not(target_os = "macos"), feature = "macos-private-api")))
+  )]
+  #[must_use]
   fn transparent(self, transparent: bool) -> Self;
 
   /// Whether the window should have borders and bars.
+  #[must_use]
   fn decorations(self, decorations: bool) -> Self;
 
   /// Whether the window should always be on top of other windows.
+  #[must_use]
   fn always_on_top(self, always_on_top: bool) -> Self;
 
   /// Sets the window icon.
   fn icon(self, icon: Icon) -> crate::Result<Self>;
 
   /// Sets whether or not the window icon should be added to the taskbar.
+  #[must_use]
   fn skip_taskbar(self, skip: bool) -> Self;
 
   /// Sets a parent to the window to be created.
@@ -166,6 +162,7 @@ pub trait WindowBuilder: WindowBuilderBase {
   ///
   /// For more information, see <https://docs.microsoft.com/en-us/windows/win32/winmsg/window-features#child-windows>
   #[cfg(windows)]
+  #[must_use]
   fn parent_window(self, parent: HWND) -> Self;
 
   /// Set an owner to the window to be created.
@@ -177,30 +174,14 @@ pub trait WindowBuilder: WindowBuilderBase {
   ///
   /// For more information, see <https://docs.microsoft.com/en-us/windows/win32/winmsg/window-features#owned-windows>
   #[cfg(windows)]
+  #[must_use]
   fn owner_window(self, owner: HWND) -> Self;
 
   /// Whether the icon was set or not.
   fn has_icon(&self) -> bool;
 
-  /// Whether the menu was set or not.
-  #[cfg(feature = "menu")]
-  #[cfg_attr(doc_cfg, doc(cfg(feature = "menu")))]
-  fn has_menu(&self) -> bool;
-}
-
-/// Rpc request.
-pub struct RpcRequest {
-  /// RPC command.
-  pub command: String,
-  /// Params.
-  pub params: Option<JsonValue>,
-}
-
-/// Uses a custom URI scheme handler to resolve file requests
-pub struct CustomProtocol {
-  /// Handler for protocol
-  #[allow(clippy::type_complexity)]
-  pub protocol: Box<dyn Fn(&str) -> Result<Vec<u8>, Box<dyn std::error::Error>> + Send + Sync>,
+  /// Gets the window menu.
+  fn get_menu(&self) -> Option<&Menu>;
 }
 
 /// The file drop event payload.
@@ -215,21 +196,9 @@ pub enum FileDropEvent {
   Cancelled,
 }
 
-/// Rpc handler.
-pub type WebviewRpcHandler<P> = Box<dyn Fn(DetachedWindow<P>, RpcRequest) + Send>;
+/// IPC handler.
+pub type WebviewIpcHandler<R> = Box<dyn Fn(DetachedWindow<R>, String) + Send>;
 
 /// File drop handler callback
 /// Return `true` in the callback to block the OS' default behavior of handling a file drop.
-pub type FileDropHandler<P> = Box<dyn Fn(FileDropEvent, DetachedWindow<P>) -> bool + Send>;
-
-#[derive(Deserialize)]
-pub struct InvokePayload {
-  #[serde(rename = "__tauriModule")]
-  pub tauri_module: Option<String>,
-  pub callback: String,
-  pub error: String,
-  #[serde(rename = "__invokeKey")]
-  pub key: u32,
-  #[serde(flatten)]
-  pub inner: JsonValue,
-}
+pub type FileDropHandler<R> = Box<dyn Fn(FileDropEvent, DetachedWindow<R>) -> bool + Send>;
